//Base class for Virtual Sample PlugIn Synths
//VS PlugIn Synths must be derived from this class

unit synthbase;

interface

Uses Windows;

Type
   Float=Single;
   TFloatArray=Array[0..1000] Of Float;
   PFloatArray=^TFloatArray;

   //PlugIn Synth
   TSynthProgramInfo=Record
      Name:Array[0..63] Of Char;
      Params:Pointer;
   End;

   PSynthLoadProgramRec=^TSynthLoadProgramRec;
   TSynthLoadProgramRec=Array[0..1000] Of TSynthProgramInfo;

   TVSSynthBase=Class
         Protected
           FSampleRate:LongInt;
         Public
           Constructor Create(ASampleRate:LongInt);Virtual;
           Destructor Destroy;Override;

           Procedure SetProgram(NewValue:LongInt);Virtual;
           Procedure GetProgramName(Index:LongInt;Var Result:String);Virtual;
           Function GetProgramCount:LongInt;Virtual;
           Function GetParamCount(GroupIndex:LongInt):LongInt;Virtual;
           Procedure GetParamLabel(GroupIndex,Index:LongInt;Var Result:String);Virtual;
           Procedure GetParamDisplay(GroupIndex,Index:LongInt;Var Result:String);Virtual;
           Procedure GetParamValue(GroupIndex,Index:LongInt;Var Result:Float);Virtual;
           Procedure SetParamValue(GroupIndex,Index:LongInt;NewValue:Float);Virtual;
           Function GetGroupCount:LongInt;Virtual;
           Procedure GetGroupName(Index:LongInt;Var Result:String);Virtual;
           Procedure NoteOn(Note,Velocity,ScaleCents:LongInt);Virtual;
           Procedure NoteOff;Virtual;
           Procedure LoadParams(Params:Pointer;ParamsLen:LongInt);Virtual;
           Procedure GetParamsInfo(Var ParamsLen:LongInt);Virtual;
           Procedure StoreParams(Params:Pointer;ParamsLen:LongInt);Virtual;
           Function GetBuffer(Buffer:PFloatArray;SampleCount:LongInt):LongInt;Virtual;
           Procedure InitializeProgram;Virtual;
           Procedure GetFileExt(Var Result:String);Virtual;
           Function SaveProgram(Var FileName:String):LongInt;Virtual;
           Procedure GetProgramInfo(Const FileName:String;Var ProgramCount,ParamsLen:LongInt);Virtual;
           Function LoadProgram(Var FileName:String;Var Res:TSynthLoadProgramRec):LongInt;Virtual;
           Procedure SetSampleRate(ASampleRate:LongInt);Virtual;
           Function HasEditor:Boolean;Virtual;
           Procedure GetEditorSize(Var Width,Height:LongInt);Virtual;
           Procedure ShowEditor(frame:HWND);Virtual;
           Procedure CloseEditor;Virtual;
         Public
           Property SampleRate:Longint read FSampleRate write FSampleRate;
   End;

Type
     TSynthSetProgram=Procedure(Instance:TVSSynthBase;NewValue:LongInt);CDecl;
     TSynthGetProgramName=Procedure(Instance:TVSSynthBase;Index:LongInt;Result:PChar);CDecl;
     TSynthGetProgramCount=Function(Instance:TVSSynthBase):LongInt;CDecl;
     TSynthSetNoteOff=Procedure(Instance:TVSSynthBase;NewValue:LongBool);CDecl;
     TSynthGetParamCount=Function(Instance:TVSSynthBase;GroupIndex:LongInt):LongInt;CDecl;
     TSynthGetParamLabel=Procedure(Instance:TVSSynthBase;GroupIndex,Index:LongInt;Result:PChar);CDecl;
     TSynthGetParamDisplay=Procedure(Instance:TVSSynthBase;GroupIndex,Index:LongInt;Result:PChar);CDecl;
     TSynthGetParamValue=Procedure(Instance:TVSSynthBase;GroupIndex,Index:LongInt;Var Result:Single);CDecl;
     TSynthSetParamValue=Procedure(Instance:TVSSynthBase;GroupIndex,Index:LongInt;NewValue:Single);CDecl;
     TSynthGetGroupCount=Function(Instance:TVSSynthBase):LongInt;CDecl;
     TSynthGetGroupName=Procedure(Instance:TVSSynthBase;Index:LongInt;Result:PChar);CDecl;
     TSynthNoteOn=Procedure(Instance:TVSSynthBase;Note,Velocity:LongInt;ScaleCents:LongInt);CDecl;
     TSynthLoadParams=Procedure(Instance:TVSSynthBase;Params:Pointer;ParamsLen:LongInt);CDecl;
     TSynthGetParamsInfo=Procedure(Instance:TVSSynthBase;Var ParamsLen:LongInt);CDecl;
     TSynthStoreParams=Procedure(Instance:TVSSynthBase;Params:Pointer;ParamsLen:LongInt);CDecl;
     TSynthGetBuffer=Function(Instance:TVSSynthBase;Buffer:PFloatArray;SampleCount:Cardinal):Cardinal;CDecl;
     TSynthInitializeProgram=Procedure(Instance:TVSSynthBase);CDecl;
     TSynthGetFileExt=Procedure(Instance:TVSSynthBase;Result:PChar);CDecl;
     TSynthSaveProgram=Function(Instance:TVSSynthBase;FileName:PChar):LongInt;CDecl;
     TSynthGetProgramInfo=Procedure(Instance:TVSSynthBase;FileName:PChar;Var ProgramCount:LongInt;Var ParamsLen:LongInt);CDecl;
     TSynthLoadProgram=Function(Instance:TVSSynthBase;FileName:PChar;Result:PSynthLoadProgramRec):LongInt;CDecl;
     TSynthSetSampleRate=Procedure(Instance:TVSSynthBase;SampleRate:LongInt);cdecl;
     TSynthHasEditor=Function(Instance:TVSSynthBase):LongBool;cdecl;
     TSynthGetEditorSize=Procedure(Instance:TVSSynthBase;Var Width,Height:LongInt);cdecl;
     TSynthShowEditor=Procedure(Instance:TVSSynthBase;frame:HWND);cdecl;
     TSynthCloseEditor=Procedure(Instance:TVSSynthBase);cdecl;

     TPlugInSynth=Record
          Name:Array[0..63] Of Char;
          Flags:LongInt; //currently 0

          SetProgram:TSynthSetProgram;
          GetProgramName:TSynthGetProgramName;
          GetProgramCount:TSynthGetProgramCount;
          SetNoteOff:TSynthSetNoteOff;
          GetParamCount:TSynthGetParamCount;
          GetParamLabel:TSynthGetParamLabel;
          GetParamDisplay:TSynthGetParamDisplay;
          GetParamValue:TSynthGetParamValue;
          SetParamValue:TSynthSetParamValue;
          GetGroupCount:TSynthGetGroupCount;
          GetGroupName:TSynthGetGroupName;
          NoteOn:TSynthNoteOn;
          LoadParams:TSynthLoadParams;
          GetParamsInfo:TSynthGetParamsInfo;
          StoreParams:TSynthStoreParams;
          GetBuffer:TSynthGetBuffer;
          InitializeProgram:TSynthInitializeProgram;
          GetFileExt:TSynthGetFileExt;
          SaveProgram:TSynthSaveProgram;
          GetProgramInfo:TSynthGetProgramInfo;
          LoadProgram:TSynthLoadProgram;
          SetSampleRate:TSynthSetSampleRate;
          HasEditor:TSynthHasEditor;
          GetEditorSize:TSynthGetEditorSize;
          ShowEditor:TSynthShowEditor;
          CloseEditor:TSynthCloseEditor;
     End;

Var SynthParams:TPlugInSynth;     

Function Float2String(Value:Float):String;
Function Int2String(Value:LongInt):String;

implementation

Uses SysUtils;

Constructor TVSSynthBase.Create(ASampleRate:LongInt);
Begin
     Inherited Create;
     FSampleRate:=ASampleRate;
End;

Destructor TVSSynthBase.Destroy;
Begin
     Inherited Destroy;
End;

Procedure TVSSynthBase.SetProgram(NewValue:LongInt);
Begin
End;

Procedure TVSSynthBase.GetProgramName(Index:LongInt;Var Result:String);
Begin
     Result:='';
End;

Function TVSSynthBase.GetProgramCount:LongInt;
Begin
     Result:=0;
End;

Function TVSSynthBase.GetParamCount(GroupIndex:LongInt):LongInt;
Begin
     Result:=0;
End;

Procedure TVSSynthBase.GetParamLabel(GroupIndex,Index:LongInt;Var Result:String);
Begin
     Result:='';
End;

Procedure TVSSynthBase.GetParamDisplay(GroupIndex,Index:LongInt;Var Result:String);
Begin
     Result:='';
End;

Procedure TVSSynthBase.GetParamValue(GroupIndex,Index:LongInt;Var Result:Float);
Begin
End;

Procedure TVSSynthBase.SetParamValue(GroupIndex,Index:LongInt;NewValue:Float);
Begin
End;

Function TVSSynthBase.GetGroupCount:LongInt;
Begin
     Result:=0;
End;

Procedure TVSSynthBase.GetGroupName(Index:LongInt;Var Result:String);
Begin
     Result:='';
End;

Procedure TVSSynthBase.NoteOn(Note,Velocity,ScaleCents:LongInt);
Begin
End;

Procedure TVSSynthBase.NoteOff;
Begin
End;

Procedure TVSSynthBase.LoadParams(Params:Pointer;ParamsLen:LongInt);
Begin
End;

Procedure TVSSynthBase.GetParamsInfo(Var ParamsLen:LongInt);
Begin
     ParamsLen:=0;
End;

Procedure TVSSynthBase.StoreParams(Params:Pointer;ParamsLen:LongInt);
Begin
End;

Function TVSSynthBase.GetBuffer(Buffer:PFloatArray;SampleCount:LongInt):LongInt;
Begin
     FillChar(Buffer^,SampleCount*sizeof(Float),0);
     Result:=0;
End;

Procedure TVSSynthBase.InitializeProgram;
Begin
End;

Procedure TVSSynthBase.GetFileExt(Var Result:String);
Begin
     Result:='';
End;

Function TVSSynthBase.SaveProgram(Var FileName:String):LongInt;
Begin
     FileName:='Not supported';
     Result:=-1;
End;

Procedure TVSSynthBase.GetProgramInfo(Const FileName:String;Var ProgramCount,ParamsLen:LongInt);
Begin
     ProgramCount:=0;
     ParamsLen:=0;
End;

Function TVSSynthBase.LoadProgram(Var FileName:String;Var Res:TSynthLoadProgramRec):LongInt;
Begin
     FileName:='Not supported';
     Result:=-1;
End;

Procedure TVSSynthBase.SetSampleRate(ASampleRate:LongInt);
Begin
     FSampleRate:=ASampleRate;
End;

Function TVSSynthBase.HasEditor:Boolean;
Begin
     Result:=False;
End;

Procedure TVSSynthBase.GetEditorSize(Var Width,Height:LongInt);
Begin
End;

Procedure TVSSynthBase.ShowEditor(frame:HWND);
Begin
End;

Procedure TVSSynthBase.CloseEditor;
Begin
End;

//Interface Functions
Procedure SynthSetProgram(Instance:TVSSynthBase;NewValue:LongInt);CDecl;
Begin
     Instance.SetProgram(NewValue);
End;

Procedure SynthGetProgramName(Instance:TVSSynthBase;Index:LongInt;Result:PChar);CDecl;
Var s:String;
Begin
     Instance.GetProgramName(Index,s);
     StrPCopy(Result,s);
End;

Function SynthGetProgramCount(Instance:TVSSynthBase):LongInt;CDecl;
Begin
     Result:=Instance.GetProgramCount;
End;

Procedure SynthSetNoteOff(Instance:TVSSynthBase;NewValue:LongBool);CDecl;
Begin
     If NewValue Then Instance.NoteOff;
End;

Function SynthGetParamCount(Instance:TVSSynthBase;GroupIndex:LongInt):LongInt;CDecl;
Begin
     Result:=Instance.GetParamCount(GroupIndex);
End;

Procedure SynthGetParamLabel(Instance:TVSSynthBase;GroupIndex,Index:LongInt;Result:PChar);CDecl;
Var s:String;
Begin
     Instance.GetParamLabel(GroupIndex,Index,s);
     StrPCopy(Result,s);
End;

Procedure SynthGetParamDisplay(Instance:TVSSynthBase;GroupIndex,Index:LongInt;Result:PChar);CDecl;
Var s:String;
Begin
     Instance.GetParamDisplay(GroupIndex,Index,s);
     StrPCopy(Result,s);
End;

Procedure SynthGetParamValue(Instance:TVSSynthBase;GroupIndex,Index:LongInt;Var Result:Single);CDecl;
Begin
     Instance.GetParamValue(GroupIndex,Index,Result);
End;

Procedure SynthSetParamValue(Instance:TVSSynthBase;GroupIndex,Index:LongInt;NewValue:Single);CDecl;
Begin
     Instance.SetParamValue(GroupIndex,Index,NewValue);
End;

Function SynthGetGroupCount(Instance:TVSSynthBase):LongInt;CDecl;
Begin
     Result:=Instance.GetGroupCount;
End;

Procedure SynthGetGroupName(Instance:TVSSynthBase;Index:LongInt;Result:PChar);CDecl;
Var s:String;
Begin
     Instance.GetGroupName(Index,s);
     StrPCopy(Result,s);
End;

Procedure SynthNoteOn(Instance:TVSSynthBase;Note,Velocity:LongInt;ScaleCents:LongInt);CDecl;
Begin
     Instance.NoteOn(Note,Velocity,ScaleCents);
End;

Procedure SynthLoadParams(Instance:TVSSynthBase;Params:Pointer;ParamsLen:LongInt);CDecl;
Begin
     Instance.LoadParams(Params,ParamsLen);
End;

Procedure SynthGetParamsInfo(Instance:TVSSynthBase;Var ParamsLen:LongInt);CDecl;
Begin
     Instance.GetParamsInfo(ParamsLen);
End;

Procedure SynthStoreParams(Instance:TVSSynthBase;Params:Pointer;ParamsLen:LongInt);CDecl;
Begin
     Instance.StoreParams(Params,ParamsLen);
End;

Function SynthGetBuffer(Instance:TVSSynthBase;Buffer:PFloatArray;SampleCount:Cardinal):Cardinal;CDecl;
Begin
     Result:=Instance.GetBuffer(Buffer,SampleCount);
End;

Procedure SynthInitializeProgram(Instance:TVSSynthBase);CDecl;
Begin
     Instance.InitializeProgram;
End;

Procedure SynthGetFileExt(Instance:TVSSynthBase;Result:PChar);CDecl;
Var s:String;
Begin
     Instance.GetFileExt(s);
     StrPCopy(Result,s);
End;

Function SynthSaveProgram(Instance:TVSSynthBase;FileName:PChar):LongInt;CDecl;
Var s:String;
Begin
     s:=StrPas(FileName);
     Result:=Instance.SaveProgram(s);
     StrPCopy(FileName,s);
End;

Procedure SynthGetProgramInfo(Instance:TVSSynthBase;FileName:PChar;Var ProgramCount:LongInt;Var ParamsLen:LongInt);CDecl;
Begin
     Instance.GetProgramInfo(FileName,ProgramCount,ParamsLen);
End;

Function SynthLoadProgram(Instance:TVSSynthBase;FileName:PChar;Res:PSynthLoadProgramRec):LongInt;CDecl;
Var s:String;
Begin
     s:=StrPas(FileName);
     Result:=Instance.LoadProgram(s,Res^);
     StrPCopy(FileName,s);
End;

Procedure SynthSetSampleRate(Instance:TVSSynthBase;SampleRate:LongInt);cdecl;
Begin
     Instance.SetSampleRate(SampleRate);
End;

Function SynthHasEditor(Instance:TVSSynthBase):LongBool;cdecl;
Begin
     Result:=Instance.HasEditor;
End;

Procedure SynthGetEditorSize(Instance:TVSSynthBase;Var Width,Height:LongInt);cdecl;
Begin
     Instance.GetEditorSize(Width,Height);
End;

Procedure SynthShowEditor(Instance:TVSSynthBase;frame:HWND);cdecl;
Begin
     Instance.ShowEditor(frame);
End;

Procedure SynthCloseEditor(Instance:TVSSynthBase);cdecl;
Begin
     Instance.CloseEditor;
End;

//Helpers
Function Float2String(Value:Float):String;
Begin
     Result:=FloatToStrF(Value,ffFixed,7,2);
End;

Function Int2String(Value:LongInt):String;
Begin
     Result:=IntToStr(Value);
End;

Initialization
   With SynthParams Do
   Begin
      Name:='Base Synth';
      Flags:=0; //currently 0

      SetProgram:=SynthSetProgram;
      GetProgramName:=SynthGetProgramName;
      GetProgramCount:=SynthGetProgramCount;
      SetNoteOff:=SynthSetNoteOff;
      GetParamCount:=SynthGetParamCount;
      GetParamLabel:=SynthGetParamLabel;
      GetParamDisplay:=SynthGetParamDisplay;
      GetParamValue:=SynthGetParamValue;
      SetParamValue:=SynthSetParamValue;
      GetGroupCount:=SynthGetGroupCount;
      GetGroupName:=SynthGetGroupName;
      NoteOn:=SynthNoteOn;
      LoadParams:=SynthLoadParams;
      GetParamsInfo:=SynthGetParamsInfo;
      StoreParams:=SynthStoreParams;
      GetBuffer:=SynthGetBuffer;
      InitializeProgram:=SynthInitializeProgram;
      GetFileExt:=SynthGetFileExt;
      SaveProgram:=SynthSaveProgram;
      GetProgramInfo:=SynthGetProgramInfo;
      LoadProgram:=SynthLoadProgram;
      SetSampleRate:=SynthSetSampleRate;
      HasEditor:=SynthHasEditor;
      GetEditorSize:=SynthGetEditorSize;
      ShowEditor:=SynthShowEditor;
      CloseEditor:=SynthCloseEditor;
   End;
End.
